@@ -23,14 +23,14 @@
 
 #include "gssapiP_krb5.h"
 
-OM_uint32
+OM_uint32 KRB5_CALLCONV
 krb5_gss_release_cred(minor_status, cred_handle)
     OM_uint32 *minor_status;
     gss_cred_id_t *cred_handle;
 {
     krb5_context context;
     krb5_gss_cred_id_t cred;
-    krb5_error_code code1, code2, code3;
+    krb5_error_code code1, code2;
 
     code1 = krb5_gss_init_context(&context);
     if (code1) {
@@ -44,12 +44,6 @@ krb5_gss_release_cred(minor_status, cred_handle)
         return(GSS_S_COMPLETE);
     }
 
-    if (! kg_delete_cred_id(*cred_handle)) {
-        *minor_status = (OM_uint32) G_VALIDATE_FAILED;
-        krb5_free_context(context);
-        return(GSS_S_CALL_BAD_STRUCTURE|GSS_S_NO_CRED);
-    }
-
     cred = (krb5_gss_cred_id_t)*cred_handle;
 
     k5_mutex_destroy(&cred->lock);
@@ -63,6 +57,9 @@ krb5_gss_release_cred(minor_status, cred_handle)
     } else
         code1 = 0;
 
+    if (cred->client_keytab)
+        krb5_kt_close(context, cred->client_keytab);
+
 #ifndef LEAN_CLIENT
     if (cred->keytab)
         code2 = krb5_kt_close(context, cred->keytab);
@@ -71,19 +68,17 @@ krb5_gss_release_cred(minor_status, cred_handle)
         code2 = 0;
 
     if (cred->rcache)
-        code3 = krb5_rc_close(context, cred->rcache);
-    else
-        code3 = 0;
+        k5_rc_close(context, cred->rcache);
     if (cred->name)
-        kg_release_name(context, 0, &cred->name);
+        kg_release_name(context, &cred->name);
+
+    krb5_free_principal(context, cred->impersonator);
 
     if (cred->req_enctypes)
         free(cred->req_enctypes);
 
-    if (cred->password.data) {
-        zap(cred->password.data, cred->password.length);
-        krb5_free_data_contents(context, &cred->password);
-    }
+    if (cred->password != NULL)
+        zapfree(cred->password, strlen(cred->password));
 
     xfree(cred);
 
@@ -94,8 +89,6 @@ krb5_gss_release_cred(minor_status, cred_handle)
         *minor_status = code1;
     if (code2)
         *minor_status = code2;
-    if (code3)
-        *minor_status = code3;
 
     if (*minor_status)
         save_error_info(*minor_status, context);
