@@ -323,6 +323,10 @@ copy_vtable(const kdb_vftabl *in, kdb_vftabl *out)
     out->refresh_config = in->refresh_config;
     out->check_allowed_to_delegate = in->check_allowed_to_delegate;
     out->free_principal_e_data = in->free_principal_e_data;
+    out->get_s4u_x509_principal = in->get_s4u_x509_principal;
+    out->allowed_to_delegate_from = in->allowed_to_delegate_from;
+    out->get_authdata_info = in->get_authdata_info;
+    out->free_authdata_info = in->free_authdata_info;
 
     /* Set defaults for optional fields. */
     if (out->fetch_master_key == NULL)
@@ -1034,8 +1038,7 @@ krb5_db_rename_principal(krb5_context kcontext, krb5_principal source,
         logging(kcontext))
         return KRB5_PLUGIN_OP_NOTSUPP;
 
-    status = krb5_db_get_principal(kcontext, target, KRB5_KDB_FLAG_ALIAS_OK,
-                                   &entry);
+    status = krb5_db_get_principal(kcontext, target, 0, &entry);
     if (status == 0) {
         krb5_db_free_principal(kcontext, entry);
         return KRB5_KDB_INUSE;
@@ -2312,15 +2315,11 @@ krb5_dbe_compute_salt(krb5_context context, const krb5_key_data *key,
         if (retval)
             return retval;
         break;
-    case KRB5_KDB_SALTTYPE_V4:
-        sdata = empty_data();
-        break;
     case KRB5_KDB_SALTTYPE_NOREALM:
         retval = krb5_principal2salt_norealm(context, princ, &sdata);
         if (retval)
             return retval;
         break;
-    case KRB5_KDB_SALTTYPE_AFS3:
     case KRB5_KDB_SALTTYPE_ONLYREALM:
         return krb5_copy_data(context, &princ->realm, salt_out);
     case KRB5_KDB_SALTTYPE_SPECIAL:
@@ -2592,11 +2591,14 @@ krb5_db_set_context(krb5_context context, void *db_context)
 
 krb5_error_code
 krb5_db_sign_authdata(krb5_context kcontext, unsigned int flags,
-                      krb5_const_principal client_princ, krb5_db_entry *client,
-                      krb5_db_entry *server, krb5_db_entry *krbtgt,
-                      krb5_keyblock *client_key, krb5_keyblock *server_key,
-                      krb5_keyblock *krbtgt_key, krb5_keyblock *session_key,
+                      krb5_const_principal client_princ,
+                      krb5_const_principal server_princ, krb5_db_entry *client,
+                      krb5_db_entry *server, krb5_db_entry *header_server,
+                      krb5_db_entry *local_tgt, krb5_keyblock *client_key,
+                      krb5_keyblock *server_key, krb5_keyblock *header_key,
+                      krb5_keyblock *local_tgt_key, krb5_keyblock *session_key,
                       krb5_timestamp authtime, krb5_authdata **tgt_auth_data,
+                      void *ad_info, krb5_data ***auth_indicators,
                       krb5_authdata ***signed_auth_data)
 {
     krb5_error_code status = 0;
@@ -2608,10 +2610,11 @@ krb5_db_sign_authdata(krb5_context kcontext, unsigned int flags,
         return status;
     if (v->sign_authdata == NULL)
         return KRB5_PLUGIN_OP_NOTSUPP;
-    return v->sign_authdata(kcontext, flags, client_princ, client, server,
-                            krbtgt, client_key, server_key, krbtgt_key,
-                            session_key, authtime, tgt_auth_data,
-                            signed_auth_data);
+    return v->sign_authdata(kcontext, flags, client_princ, server_princ,
+                            client, server, header_server, local_tgt,
+                            client_key, server_key, header_key, local_tgt_key,
+                            session_key, authtime, tgt_auth_data, ad_info,
+                            auth_indicators, signed_auth_data);
 }
 
 krb5_error_code
@@ -2717,6 +2720,93 @@ krb5_db_check_allowed_to_delegate(krb5_context kcontext,
     return v->check_allowed_to_delegate(kcontext, client, server, proxy);
 }
 
+krb5_error_code
+krb5_db_get_s4u_x509_principal(krb5_context kcontext,
+                               const krb5_data *client_cert,
+                               krb5_const_principal in_princ,
+                               unsigned int flags, krb5_db_entry **entry)
+{
+    krb5_error_code ret;
+    kdb_vftabl *v;
+
+    ret = get_vftabl(kcontext, &v);
+    if (ret)
+        return ret;
+    if (v->get_s4u_x509_principal == NULL)
+        return KRB5_PLUGIN_OP_NOTSUPP;
+    ret = v->get_s4u_x509_principal(kcontext, client_cert, in_princ, flags,
+                                    entry);
+    if (ret)
+        return ret;
+
+    /* Sort the keys in the db entry, same as get_principal(). */
+    if ((*entry)->key_data != NULL)
+        krb5_dbe_sort_key_data((*entry)->key_data, (*entry)->n_key_data);
+
+    return 0;
+}
+
+krb5_error_code
+krb5_db_allowed_to_delegate_from(krb5_context kcontext,
+                                 krb5_const_principal client,
+                                 krb5_const_principal server,
+                                 void *server_ad_info,
+                                 const krb5_db_entry *proxy)
+{
+    krb5_error_code ret;
+    kdb_vftabl *v;
+
+    ret = get_vftabl(kcontext, &v);
+    if (ret)
+        return ret;
+    if (v->allowed_to_delegate_from == NULL)
+        return KRB5_PLUGIN_OP_NOTSUPP;
+    return v->allowed_to_delegate_from(kcontext, client, server,
+                                       server_ad_info, proxy);
+}
+
+krb5_error_code
+krb5_db_get_authdata_info(krb5_context kcontext, unsigned int flags,
+                          krb5_authdata **in_authdata,
+                          krb5_const_principal client_princ,
+                          krb5_const_principal server_princ,
+                          krb5_keyblock *server_key, krb5_keyblock *krbtgt_key,
+                          krb5_db_entry *krbtgt, krb5_timestamp authtime,
+                          void **ad_info_out, krb5_principal *client_out)
+{
+    krb5_error_code ret;
+    kdb_vftabl *v;
+
+    *ad_info_out = NULL;
+    if (client_out != NULL)
+        *client_out = NULL;
+
+    ret = get_vftabl(kcontext, &v);
+    if (ret)
+        return ret;
+    if (v->get_authdata_info == NULL)
+        return KRB5_PLUGIN_OP_NOTSUPP;
+    return v->get_authdata_info(kcontext, flags, in_authdata, client_princ,
+                                server_princ, server_key, krbtgt_key, krbtgt,
+                                authtime, ad_info_out, client_out);
+}
+
+void
+krb5_db_free_authdata_info(krb5_context kcontext, void *ad_info)
+{
+    krb5_error_code ret;
+    kdb_vftabl *v;
+
+    if (ad_info == NULL)
+        return;
+    ret = get_vftabl(kcontext, &v);
+    if (ret)
+        return;
+    if (v->free_authdata_info == NULL)
+        return;
+    v->free_authdata_info(kcontext, ad_info);
+}
+
 void
 krb5_dbe_sort_key_data(krb5_key_data *key_data, size_t key_data_length)
 {
