@@ -691,15 +691,6 @@ dcc_ptcursor_free(krb5_context context, krb5_cc_ptcursor *cursor)
     return 0;
 }
 
-static krb5_error_code KRB5_CALLCONV
-dcc_lastchange(krb5_context context, krb5_ccache cache,
-               krb5_timestamp *time_out)
-{
-    dcc_data *data = cache->data;
-
-    return krb5_fcc_ops.lastchange(context, data->fcc, time_out);
-}
-
 static krb5_error_code KRB5_CALLCONV
 dcc_lock(krb5_context context, krb5_ccache cache)
 {
@@ -762,7 +753,6 @@ const krb5_cc_ops krb5_dcc_ops = {
     dcc_ptcursor_next,
     dcc_ptcursor_free,
     NULL, /* move */
-    dcc_lastchange,
     NULL, /* wasdefault */
     dcc_lock,
     dcc_unlock,
