@@ -84,9 +84,6 @@ static krb5_error_code KRB5_CALLCONV krb5_mcc_ptcursor_next
 static krb5_error_code KRB5_CALLCONV krb5_mcc_ptcursor_free
 (krb5_context, krb5_cc_ptcursor *);
 
-static krb5_error_code KRB5_CALLCONV krb5_mcc_last_change_time
-(krb5_context, krb5_ccache, krb5_timestamp *);
-
 static krb5_error_code KRB5_CALLCONV krb5_mcc_lock
 (krb5_context context, krb5_ccache id);
 
@@ -111,7 +108,6 @@ typedef struct _krb5_mcc_data {
     k5_cc_mutex lock;
     krb5_principal prin;
     krb5_mcc_link *link;
-    krb5_timestamp changetime;
     /* Time offsets for clock-skewed clients.  */
     krb5_int32 time_offset;
     krb5_int32 usec_offset;
@@ -133,8 +129,6 @@ struct krb5_mcc_ptcursor_data {
 k5_cc_mutex krb5int_mcc_mutex = K5_CC_MUTEX_PARTIAL_INITIALIZER;
 static struct k5_hashtab *mcc_hashtab = NULL;
 
-static void update_mcc_change_time(krb5_mcc_data *);
-
 /* Ensure that mcc_hashtab is initialized.  Call with krb5int_mcc_mutex
  * locked. */
 static krb5_error_code
@@ -192,7 +186,6 @@ krb5_mcc_initialize(krb5_context context, krb5_ccache id, krb5_principal princ)
     empty_mcc_cache(context, d);
 
     ret = krb5_copy_principal(context, princ, &d->prin);
-    update_mcc_change_time(d);
 
     if (os_ctx->os_flags & KRB5_OS_TOFFSET_VALID) {
         /* Store client time offsets in the cache */
@@ -405,14 +398,23 @@ krb5_mcc_next_cred(krb5_context context, krb5_ccache id,
      */
     k5_cc_mutex_lock(context, &d->lock);
     if (mcursor->generation != d->generation) {
-        k5_cc_mutex_unlock(context, &d->lock);
-        return KRB5_CC_END;
+        retval = KRB5_CC_END;
+        goto done;
+    }
+
+    /* Skip over removed creds. */
+    while (mcursor->next_link != NULL && mcursor->next_link->creds == NULL)
+        mcursor->next_link = mcursor->next_link->next;
+    if (mcursor->next_link == NULL) {
+        retval = KRB5_CC_END;
+        goto done;
     }
 
     retval = k5_copy_creds_contents(context, mcursor->next_link->creds, creds);
     if (retval == 0)
         mcursor->next_link = mcursor->next_link->next;
 
+done:
     k5_cc_mutex_unlock(context, &d->lock);
     return retval;
 }
@@ -469,12 +471,10 @@ new_mcc_data (const char *name, krb5_mcc_data **dataptr)
     }
     d->link = NULL;
     d->prin = NULL;
-    d->changetime = 0;
     d->time_offset = 0;
     d->usec_offset = 0;
     d->refcount = 2;
     d->generation = 0;
-    update_mcc_change_time(d);
 
     if (k5_hashtab_add(mcc_hashtab, d->name, strlen(d->name), d) != 0) {
         free(d->name);
@@ -592,16 +592,31 @@ krb5_mcc_retrieve(krb5_context context, krb5_ccache id, krb5_flags whichfields,
 }
 
 /*
- * Non-functional stub implementation for krb5_mcc_remove
+ * Modifies:
+ * the memory cache
  *
- * Errors:
- *    KRB5_CC_NOSUPP - not implemented
+ * Effects:
+ * Remove the given creds from the ccache.
  */
 static krb5_error_code KRB5_CALLCONV
 krb5_mcc_remove_cred(krb5_context context, krb5_ccache cache, krb5_flags flags,
                      krb5_creds *creds)
 {
-    return KRB5_CC_NOSUPP;
+    krb5_mcc_data *data = (krb5_mcc_data *)cache->data;
+    krb5_mcc_link *l;
+
+    k5_cc_mutex_lock(context, &data->lock);
+
+    for (l = data->link; l != NULL; l = l->next) {
+        if (l->creds != NULL &&
+            krb5int_cc_creds_match_request(context, flags, creds, l->creds)) {
+            krb5_free_creds(context, l->creds);
+            l->creds = NULL;
+        }
+    }
+
+    k5_cc_mutex_unlock(context, &data->lock);
+    return 0;
 }
 
 
@@ -656,7 +671,6 @@ krb5_mcc_store(krb5_context ctx, krb5_ccache id, krb5_creds *creds)
     k5_cc_mutex_lock(ctx, &mptr->lock);
     new_node->next = mptr->link;
     mptr->link = new_node;
-    update_mcc_change_time(mptr);
     k5_cc_mutex_unlock(ctx, &mptr->lock);
     return 0;
 cleanup:
@@ -725,33 +739,6 @@ krb5_mcc_ptcursor_free(
     return 0;
 }
 
-static krb5_error_code KRB5_CALLCONV
-krb5_mcc_last_change_time(
-    krb5_context context,
-    krb5_ccache id,
-    krb5_timestamp *change_time)
-{
-    krb5_mcc_data *data = (krb5_mcc_data *) id->data;
-
-    k5_cc_mutex_lock(context, &data->lock);
-    *change_time = data->changetime;
-    k5_cc_mutex_unlock(context, &data->lock);
-    return 0;
-}
-
-/*
-  Utility routine: called by krb5_mcc_* functions to keep
-  result of krb5_mcc_last_change_time up to date
-*/
-
-static void
-update_mcc_change_time(krb5_mcc_data *d)
-{
-    krb5_timestamp now_time = time(NULL);
-    d->changetime = ts_after(now_time, d->changetime) ?
-        now_time : ts_incr(d->changetime, 1);
-}
-
 static krb5_error_code KRB5_CALLCONV
 krb5_mcc_lock(krb5_context context, krb5_ccache id)
 {
@@ -792,7 +779,6 @@ const krb5_cc_ops krb5_mcc_ops = {
     krb5_mcc_ptcursor_next,
     krb5_mcc_ptcursor_free,
     NULL, /* move */
-    krb5_mcc_last_change_time,
     NULL, /* wasdefault */
     krb5_mcc_lock,
     krb5_mcc_unlock,
