@@ -27,75 +27,19 @@
 #include "k5-int.h"
 #include "int-proto.h"
 
-/*
- * Routines to deal with externalizing the krb5_context:
- *      krb5_context_size();
- *      krb5_context_externalize();
- *      krb5_context_internalize();
- *
- * Routines to deal with externalizing the krb5_os_context:
- *      krb5_oscontext_size();
- *      krb5_oscontext_externalize();
- *      krb5_oscontext_internalize();
- *
- * Routines to deal with externalizing the profile.
- *      profile_ser_size();
- *      profile_ser_externalize();
- *      profile_ser_internalize();
- *
- * Interface to initialize serializing of krb5_context and krb5_os_context:
- *      krb5_ser_context_init();
- */
-static krb5_error_code
-krb5_context_size(krb5_context, krb5_pointer, size_t *);
-
-static krb5_error_code
-krb5_context_externalize(krb5_context, krb5_pointer, krb5_octet **, size_t *);
-
-static krb5_error_code
-krb5_context_internalize(krb5_context, krb5_pointer *, krb5_octet **, size_t *);
-
-static krb5_error_code
-krb5_oscontext_size(krb5_context, krb5_pointer, size_t *);
-
-static krb5_error_code
-krb5_oscontext_externalize(krb5_context, krb5_pointer, krb5_octet **, size_t *);
-
-static krb5_error_code
-krb5_oscontext_internalize(krb5_context, krb5_pointer *,
-                           krb5_octet **, size_t *);
-
-#ifndef LEAN_CLIENT
-krb5_error_code profile_ser_size(krb5_context, krb5_pointer, size_t *);
-
-krb5_error_code profile_ser_externalize(krb5_context, krb5_pointer,
+krb5_error_code profile_ser_size(krb5_context, profile_t, size_t *);
+krb5_error_code profile_ser_externalize(krb5_context, profile_t,
+                                        krb5_octet **, size_t *);
+krb5_error_code profile_ser_internalize(krb5_context, profile_t *,
                                         krb5_octet **, size_t *);
 
-krb5_error_code profile_ser_internalize(krb5_context, krb5_pointer *,
-                                        krb5_octet **, size_t *);
-#endif /* LEAN_CLIENT */
-
-/* Local data */
-static const krb5_ser_entry krb5_context_ser_entry = {
-    KV5M_CONTEXT,                       /* Type                 */
-    krb5_context_size,                  /* Sizer routine        */
-    krb5_context_externalize,           /* Externalize routine  */
-    krb5_context_internalize            /* Internalize routine  */
-};
-static const krb5_ser_entry krb5_oscontext_ser_entry = {
-    KV5M_OS_CONTEXT,                    /* Type                 */
-    krb5_oscontext_size,                /* Sizer routine        */
-    krb5_oscontext_externalize,         /* Externalize routine  */
-    krb5_oscontext_internalize          /* Internalize routine  */
-};
-#ifndef LEAN_CLIENT
-static const krb5_ser_entry krb5_profile_ser_entry = {
-    PROF_MAGIC_PROFILE,                 /* Type                 */
-    profile_ser_size,                   /* Sizer routine        */
-    profile_ser_externalize,            /* Externalize routine  */
-    profile_ser_internalize             /* Internalize routine  */
-};
-#endif /* LEAN_CLIENT */
+static krb5_error_code size_oscontext(krb5_os_context os_ctx, size_t *sizep);
+static krb5_error_code externalize_oscontext(krb5_os_context os_ctx,
+                                             krb5_octet **buffer,
+                                             size_t *lenremain);
+static krb5_error_code internalize_oscontext(krb5_os_context *argp,
+                                             krb5_octet **buffer,
+                                             size_t *lenremain);
 
 static inline unsigned int
 etypes_len(krb5_enctype *list)
@@ -103,16 +47,11 @@ etypes_len(krb5_enctype *list)
     return (list == NULL) ? 0 : k5_count_etypes(list);
 }
 
-/*
- * krb5_context_size()  - Determine the size required to externalize the
- *                        krb5_context.
- */
-static krb5_error_code
-krb5_context_size(krb5_context kcontext, krb5_pointer arg, size_t *sizep)
+krb5_error_code
+k5_size_context(krb5_context context, size_t *sizep)
 {
     krb5_error_code     kret;
     size_t              required;
-    krb5_context        context;
 
     /*
      * The KRB5 context itself requires:
@@ -124,60 +63,41 @@ krb5_context_size(krb5_context kcontext, krb5_pointer arg, size_t *sizep)
      *  krb5_int32                      for n_tgs_etypes*sizeof(krb5_int32)
      *  nktypes*sizeof(krb5_int32)      for tgs_etypes.
      *  krb5_int32                      for clockskew
-     *  krb5_int32                      for kdc_req_sumtype
-     *  krb5_int32                      for ap_req_sumtype
-     *  krb5_int32                      for safe_sumtype
      *  krb5_int32                      for kdc_default_options
      *  krb5_int32                      for library_options
      *  krb5_int32                      for profile_secure
      *  krb5_int32                      for fcc_default_format
      *    <>                            for os_context
-     *    <>                            for db_context
      *    <>                            for profile
      *  krb5_int32                      for trailer.
      */
     kret = EINVAL;
-    if ((context = (krb5_context) arg)) {
+    if (context != NULL) {
         /* Calculate base length */
-        required = (14 * sizeof(krb5_int32) +
+        required = (10 * sizeof(krb5_int32) +
                     (etypes_len(context->in_tkt_etypes) * sizeof(krb5_int32)) +
                     (etypes_len(context->tgs_etypes) * sizeof(krb5_int32)));
 
         if (context->default_realm)
             required += strlen(context->default_realm);
+
         /* Calculate size required by os_context, if appropriate */
-        kret = krb5_size_opaque(kcontext,
-                                KV5M_OS_CONTEXT,
-                                (krb5_pointer) &context->os_context,
-                                &required);
+        kret = size_oscontext(&context->os_context, &required);
 
-        /* Calculate size required by db_context, if appropriate */
-        if (!kret && context->dal_handle)
-            kret = krb5_size_opaque(kcontext,
-                                    KV5M_DB_CONTEXT,
-                                    (krb5_pointer) context->dal_handle,
-                                    &required);
-
-        /* Finally, calculate size required by profile, if appropriate */
+        /* Calculate size required by profile, if appropriate */
         if (!kret && context->profile)
-            kret = krb5_size_opaque(kcontext,
-                                    PROF_MAGIC_PROFILE,
-                                    (krb5_pointer) context->profile,
-                                    &required);
+            kret = profile_ser_size(NULL, context->profile, &required);
     }
     if (!kret)
         *sizep += required;
     return(kret);
 }
 
-/*
- * krb5_context_externalize()   - Externalize the krb5_context.
- */
-static krb5_error_code
-krb5_context_externalize(krb5_context kcontext, krb5_pointer arg, krb5_octet **buffer, size_t *lenremain)
+krb5_error_code
+k5_externalize_context(krb5_context context,
+                       krb5_octet **buffer, size_t *lenremain)
 {
     krb5_error_code     kret;
-    krb5_context        context;
     size_t              required;
     krb5_octet          *bp;
     size_t              remain;
@@ -186,13 +106,12 @@ krb5_context_externalize(krb5_context kcontext, krb5_pointer arg, krb5_octet **b
     required = 0;
     bp = *buffer;
     remain = *lenremain;
-    context = (krb5_context) arg;
     if (!context)
         return (EINVAL);
     if (context->magic != KV5M_CONTEXT)
         return (KV5M_CONTEXT);
 
-    if ((kret = krb5_context_size(kcontext, arg, &required)))
+    if ((kret = k5_size_context(context, &required)))
         return (kret);
 
     if (required > remain)
@@ -255,24 +174,6 @@ krb5_context_externalize(krb5_context kcontext, krb5_pointer arg, krb5_octet **b
     if (kret)
         return (kret);
 
-    /* Now kdc_req_sumtype */
-    kret = krb5_ser_pack_int32((krb5_int32) context->kdc_req_sumtype,
-                               &bp, &remain);
-    if (kret)
-        return (kret);
-
-    /* Now default ap_req_sumtype */
-    kret = krb5_ser_pack_int32((krb5_int32) context->default_ap_req_sumtype,
-                               &bp, &remain);
-    if (kret)
-        return (kret);
-
-    /* Now default safe_sumtype */
-    kret = krb5_ser_pack_int32((krb5_int32) context->default_safe_sumtype,
-                               &bp, &remain);
-    if (kret)
-        return (kret);
-
     /* Now kdc_default_options */
     kret = krb5_ser_pack_int32((krb5_int32) context->kdc_default_options,
                                &bp, &remain);
@@ -298,26 +199,13 @@ krb5_context_externalize(krb5_context kcontext, krb5_pointer arg, krb5_octet **b
         return (kret);
 
     /* Now handle os_context, if appropriate */
-    kret = krb5_externalize_opaque(kcontext, KV5M_OS_CONTEXT,
-                                   (krb5_pointer) &context->os_context,
-                                   &bp, &remain);
+    kret = externalize_oscontext(&context->os_context, &bp, &remain);
     if (kret)
         return (kret);
 
-    /* Now handle database context, if appropriate */
-    if (context->dal_handle) {
-        kret = krb5_externalize_opaque(kcontext, KV5M_DB_CONTEXT,
-                                       (krb5_pointer) context->dal_handle,
-                                       &bp, &remain);
-        if (kret)
-            return (kret);
-    }
-
     /* Finally, handle profile, if appropriate */
-    if (context->profile) {
-        kret = krb5_externalize_opaque(kcontext, PROF_MAGIC_PROFILE,
-                                       (krb5_pointer) context->profile,
-                                       &bp, &remain);
+    if (context->profile != NULL) {
+        kret = profile_ser_externalize(NULL, context->profile, &bp, &remain);
         if (kret)
             return (kret);
     }
@@ -336,11 +224,9 @@ krb5_context_externalize(krb5_context kcontext, krb5_pointer arg, krb5_octet **b
     return (0);
 }
 
-/*
- * krb5_context_internalize()   - Internalize the krb5_context.
- */
-static krb5_error_code
-krb5_context_internalize(krb5_context kcontext, krb5_pointer *argp, krb5_octet **buffer, size_t *lenremain)
+krb5_error_code
+k5_internalize_context(krb5_context *argp,
+                       krb5_octet **buffer, size_t *lenremain)
 {
     krb5_error_code     kret;
     krb5_context        context;
@@ -421,26 +307,11 @@ krb5_context_internalize(krb5_context kcontext, krb5_pointer *argp, krb5_octet *
     } else
         context->tgs_etypes = NULL;
 
-    /* Allowable checksum */
+    /* Allowable clockskew */
     if ((kret = krb5_ser_unpack_int32(&ibuf, &bp, &remain)))
         goto cleanup;
     context->clockskew = (krb5_deltat) ibuf;
 
-    /* kdc_req_sumtype */
-    if ((kret = krb5_ser_unpack_int32(&ibuf, &bp, &remain)))
-        goto cleanup;
-    context->kdc_req_sumtype = (krb5_cksumtype) ibuf;
-
-    /* default ap_req_sumtype */
-    if ((kret = krb5_ser_unpack_int32(&ibuf, &bp, &remain)))
-        goto cleanup;
-    context->default_ap_req_sumtype = (krb5_cksumtype) ibuf;
-
-    /* default_safe_sumtype */
-    if ((kret = krb5_ser_unpack_int32(&ibuf, &bp, &remain)))
-        goto cleanup;
-    context->default_safe_sumtype = (krb5_cksumtype) ibuf;
-
     /* kdc_default_options */
     if ((kret = krb5_ser_unpack_int32(&ibuf, &bp, &remain)))
         goto cleanup;
@@ -466,9 +337,7 @@ krb5_context_internalize(krb5_context kcontext, krb5_pointer *argp, krb5_octet *
        a pointer.  */
     {
         krb5_os_context osp = 0;
-        kret = krb5_internalize_opaque(kcontext, KV5M_OS_CONTEXT,
-                                       (krb5_pointer *) &osp,
-                                       &bp, &remain);
+        kret = internalize_oscontext(&osp, &bp, &remain);
         if (kret && (kret != EINVAL) && (kret != ENOENT))
             goto cleanup;
         /* Put the newly allocated data into the krb5_context
@@ -478,17 +347,8 @@ krb5_context_internalize(krb5_context kcontext, krb5_pointer *argp, krb5_octet *
         free(osp);
     }
 
-    /* Attempt to read in the db_context */
-    kret = krb5_internalize_opaque(kcontext, KV5M_DB_CONTEXT,
-                                   (krb5_pointer *) &context->dal_handle,
-                                   &bp, &remain);
-    if (kret && (kret != EINVAL) && (kret != ENOENT))
-        goto cleanup;
-
     /* Attempt to read in the profile */
-    kret = krb5_internalize_opaque(kcontext, PROF_MAGIC_PROFILE,
-                                   (krb5_pointer *) &context->profile,
-                                   &bp, &remain);
+    kret = profile_ser_internalize(NULL, &context->profile, &bp, &remain);
     if (kret && (kret != EINVAL) && (kret != ENOENT))
         goto cleanup;
 
@@ -504,7 +364,7 @@ krb5_context_internalize(krb5_context kcontext, krb5_pointer *argp, krb5_octet *
     context->magic = KV5M_CONTEXT;
     *buffer = bp;
     *lenremain = remain;
-    *argp = (krb5_pointer) context;
+    *argp = context;
 
     return 0;
 
@@ -514,12 +374,8 @@ cleanup:
     return(kret);
 }
 
-/*
- * krb5_oscontext_size()        - Determine the size required to externalize
- *                                the krb5_os_context.
- */
-static krb5_error_code
-krb5_oscontext_size(krb5_context kcontext, krb5_pointer arg, size_t *sizep)
+krb5_error_code
+size_oscontext(krb5_os_context os_ctx, size_t *sizep)
 {
     /*
      * We need five 32-bit integers:
@@ -530,14 +386,11 @@ krb5_oscontext_size(krb5_context kcontext, krb5_pointer arg, size_t *sizep)
     return(0);
 }
 
-/*
- * krb5_oscontext_externalize() - Externalize the krb5_os_context.
- */
-static krb5_error_code
-krb5_oscontext_externalize(krb5_context kcontext, krb5_pointer arg, krb5_octet **buffer, size_t *lenremain)
+krb5_error_code
+externalize_oscontext(krb5_os_context os_ctx,
+                      krb5_octet **buffer, size_t *lenremain)
 {
     krb5_error_code     kret;
-    krb5_os_context     os_ctx;
     size_t              required;
     krb5_octet          *bp;
     size_t              remain;
@@ -546,10 +399,9 @@ krb5_oscontext_externalize(krb5_context kcontext, krb5_pointer arg, krb5_octet *
     bp = *buffer;
     remain = *lenremain;
     kret = EINVAL;
-    if ((os_ctx = (krb5_os_context) arg)) {
+    if (os_ctx != NULL) {
         kret = ENOMEM;
-        if (!krb5_oscontext_size(kcontext, arg, &required) &&
-            (required <= remain)) {
+        if (!size_oscontext(os_ctx, &required) && required <= remain) {
             (void) krb5_ser_pack_int32(KV5M_OS_CONTEXT, &bp, &remain);
             (void) krb5_ser_pack_int32(os_ctx->time_offset, &bp, &remain);
             (void) krb5_ser_pack_int32(os_ctx->usec_offset, &bp, &remain);
@@ -567,11 +419,9 @@ krb5_oscontext_externalize(krb5_context kcontext, krb5_pointer arg, krb5_octet *
     return(kret);
 }
 
-/*
- * krb5_oscontext_internalize() - Internalize the krb5_os_context.
- */
 static krb5_error_code
-krb5_oscontext_internalize(krb5_context kcontext, krb5_pointer *argp, krb5_octet **buffer, size_t *lenremain)
+internalize_oscontext(krb5_os_context *argp,
+                      krb5_octet **buffer, size_t *lenremain)
 {
     krb5_error_code     kret;
     krb5_os_context     os_ctx;
@@ -611,7 +461,7 @@ krb5_oscontext_internalize(krb5_context kcontext, krb5_pointer *argp, krb5_octet
         }
     }
     if (!kret) {
-        *argp = (krb5_pointer) os_ctx;
+        *argp = os_ctx;
     }
     else {
         if (os_ctx)
@@ -619,20 +469,3 @@ krb5_oscontext_internalize(krb5_context kcontext, krb5_pointer *argp, krb5_octet
     }
     return(kret);
 }
-
-/*
- * Register the context serializers.
- */
-krb5_error_code KRB5_CALLCONV
-krb5_ser_context_init(krb5_context kcontext)
-{
-    krb5_error_code     kret;
-    kret = krb5_register_serializer(kcontext, &krb5_context_ser_entry);
-    if (!kret)
-        kret = krb5_register_serializer(kcontext, &krb5_oscontext_ser_entry);
-#ifndef LEAN_CLIENT
-    if (!kret)
-        kret = krb5_register_serializer(kcontext, &krb5_profile_ser_entry);
-#endif /* LEAN_CLIENT */
-    return(kret);
-}
