@@ -56,7 +56,6 @@ krb5_ldap_create(krb5_context context, char *conf_section, char **db_args)
     krb5_ldap_realm_params *rparams = NULL;
     krb5_ldap_context *ldap_context=NULL;
     krb5_boolean realm_obj_created = FALSE;
-    krb5_boolean krbcontainer_obj_created = FALSE;
     int mask = 0;
 
     /* Clear the global error string */
@@ -70,15 +69,13 @@ krb5_ldap_create(krb5_context context, char *conf_section, char **db_args)
 
     status = krb5_ldap_parse_db_params(context, db_args);
     if (status) {
-        prepend_err_str(context, _("Error processing LDAP DB params:"),
-                        status, status);
+        k5_prependmsg(context, status, _("Error processing LDAP DB params"));
         goto cleanup;
     }
 
     status = krb5_ldap_read_server_params(context, conf_section, KRB5_KDB_SRV_TYPE_ADMIN);
     if (status) {
-        prepend_err_str(context, _("Error reading LDAP server params:"),
-                        status, status);
+        k5_prependmsg(context, status, _("Error reading LDAP server params"));
         goto cleanup;
     }
     status = krb5_ldap_db_init(context, ldap_context);
@@ -123,15 +120,6 @@ krb5_ldap_create(krb5_context context, char *conf_section, char **db_args)
         goto cleanup;
 
 cleanup:
-    /* If the krbcontainer/realm creation is not complete, do the roll-back here */
-    if ((krbcontainer_obj_created) && (!realm_obj_created)) {
-        int rc;
-        rc = krb5_ldap_delete_krbcontainer(context,
-                                           ldap_context->container_dn);
-        k5_setmsg(context, rc, _("could not complete roll-back, error "
-                                 "deleting Kerberos Container"));
-    }
-
     if (rparams)
         krb5_ldap_free_realm_params(rparams);
 
