@@ -3,14 +3,6 @@
 #include <sys/types.h>
 #include <winsock2.h>
 #include "leashdll.h"
-#ifndef NO_KRB4
-#include <KerberosIV/krb.h>
-#include <prot.h>
-#else
-/* General definitions */
-#define         KSUCCESS        0
-#define         KFAILURE        255
-#endif
 #include <time.h>
 
 #include <leashwin.h>
@@ -18,8 +10,6 @@
 #include "leash-int.h"
 #include "leashids.h"
 
-#include <mitwhich.h>
-
 #include "reminder.h"
 
 static char FAR *err_context;
@@ -71,8 +61,6 @@ leash_error_message(
     int size = sizeof(message) - 1; /* -1 to leave room for NULL terminator */
     int n;
 
-    // XXX: ignore AFS for now.
-
     if (!rc5 && !rcL)
         return 0;
 
@@ -279,19 +267,11 @@ Leash_changepwd_v5(
     if ( !pkrb5_init_context )
         goto cleanup;
 
-   if (rc = pkrb5_init_context(&context)) {
-#if 0
-       com_err(argv[0], ret, "initializing kerberos library");
-#endif
+   if (rc = pkrb5_init_context(&context))
        goto cleanup;
-   }
 
-   if (rc = pkrb5_parse_name(context, principal, &princ)) {
-#if 0
-       com_err(argv[0], ret, "parsing client name");
-#endif
+   if (rc = pkrb5_parse_name(context, principal, &princ))
        goto cleanup;
-   }
 
    pkrb5_get_init_creds_opt_init(&opts);
    pkrb5_get_init_creds_opt_set_tkt_life(&opts, 5*60);
@@ -305,29 +285,13 @@ Leash_changepwd_v5(
 
 
    if (rc = pkrb5_get_init_creds_password(context, &creds, princ, password,
-                                          0, 0, 0, "kadmin/changepw", &opts)) {
-       if (rc == KRB5KRB_AP_ERR_BAD_INTEGRITY) {
-#if 0
-           com_err(argv[0], 0,
-                   "Password incorrect while getting initial ticket");
-#endif
-       }
-       else {
-#if 0
-           com_err(argv[0], ret, "getting initial ticket");
-#endif
-       }
+                                          0, 0, 0, "kadmin/changepw", &opts))
        goto cleanup;
-   }
 
    if (rc = pkrb5_change_password(context, &creds, newpassword,
                                   &result_code, &result_code_string,
-                                  &result_string)) {
-#if 0
-       com_err(argv[0], ret, "changing password");
-#endif
+                                  &result_string))
        goto cleanup;
-   }
 
    if (result_code) {
        int len = result_code_string.length +
@@ -583,23 +547,6 @@ Leash_int_kinit_ex(
                             addressless,
                             publicip
                             );
-#ifndef NO_AFS
-    if ( !rc5 ) {
-        char c;
-        char *r;
-        char *t;
-        for ( r=realm, t=temp; c=*r; r++,t++ )
-            *t = isupper(c) ? tolower(c) : c;
-        *t = '\0';
-
-        rcA = Leash_afs_klog("afs", temp, "", lifetime);
-        rcB = Leash_afs_klog("afs", "", "", lifetime);
-        if (!(rcA && rcB))
-            rcA = 0;
-        else if (!rcA)
-            rcA = rcB;
-    }
-#endif /* NO_AFS */
     custom_msg = (rc5 == KRB5KRB_AP_ERR_BAD_INTEGRITY) ? "Password incorrect" : NULL;
     return leash_error_message("Ticket initialization failed.",
                                rcL, rc5, rcA, custom_msg,
@@ -612,15 +559,6 @@ Leash_renew(void)
     if ( hKrb5 && !LeashKRB5_renew() ) {
         int lifetime;
         lifetime = Leash_get_default_lifetime() / 5;
-#ifndef NO_AFS
-        {
-            TicketList * list = NULL, * token;
-            not_an_API_LeashAFSGetToken(NULL,&list,NULL);
-            for ( token = list ; token ; token = token->next )
-                Leash_afs_klog("afs", token->realm, "", lifetime);
-            not_an_API_LeashFreeTicketList(&list);
-        }
-#endif /* NO_AFS */
         return 1;
     }
     return 0;
@@ -789,57 +727,6 @@ Leash_import(void)
     if ( Leash_ms2mit(1) ) {
         int lifetime;
         lifetime = Leash_get_default_lifetime() / 5;
-#ifndef NO_AFS
-        {
-            char c;
-            char *r;
-            char *t;
-            char  cell[256];
-            char  realm[256];
-            int   i = 0;
-            int   rcA = 0;
-            int   rcB = 0;
-
-            krb5_context ctx = 0;
-            krb5_error_code code = 0;
-            krb5_ccache cc = 0;
-            krb5_principal me = 0;
-
-            if ( !pkrb5_init_context )
-                goto cleanup;
-
-            code = pkrb5_init_context(&ctx);
-            if (code) goto cleanup;
-
-            code = pkrb5_cc_default(ctx, &cc);
-            if (code) goto cleanup;
-
-            if (code = pkrb5_cc_get_principal(ctx, cc, &me))
-                goto cleanup;
-
-            for ( r=realm, t=cell, i=0; i<krb5_princ_realm(ctx, me)->length; r++,t++,i++ ) {
-                c = krb5_princ_realm(ctx, me)->data[i];
-                *r = c;
-                *t = isupper(c) ? tolower(c) : c;
-            }
-            *r = *t = '\0';
-
-            rcA = Leash_afs_klog("afs", cell, "", lifetime);
-            rcB = Leash_afs_klog("afs", "", "", lifetime);
-            if (!(rcA && rcB))
-                rcA = 0;
-            else if (!rcA)
-                rcA = rcB;
-
-          cleanup:
-            if (me)
-                pkrb5_free_principal(ctx, me);
-            if (cc)
-                pkrb5_cc_close(ctx, cc);
-            if (ctx)
-                pkrb5_free_context(ctx);
-        }
-#endif /* NO_AFS */
         return 1;
     }
     return 0;
@@ -848,45 +735,14 @@ Leash_import(void)
 long
 Leash_kdestroy(void)
 {
-    Leash_afs_unlog();
     Leash_krb5_kdestroy();
 
     return 0;
 }
 
-long FAR
-not_an_API_LeashFreeTicketList(TicketList** ticketList)
-{
-    TicketList* tempList = *ticketList, *killList;
-
-    //if (tempList == NULL)
-    //return -1;
-
-    while (tempList)
-    {
-        killList = tempList;
-
-        tempList = (TicketList*)tempList->next;
-        free(killList->service);
-        if (killList->encTypes)
-            free(killList->encTypes);
-        free(killList);
-    }
-
-    *ticketList = NULL;
-    return 0;
-}
-
-long
-not_an_API_LeashKRB4GetTickets(TICKETINFO FAR* ticketinfo,
-                               TicketList** ticketList)
-{
-    return(KFAILURE);
-}
-
 long FAR Leash_klist(HWND hlist, TICKETINFO FAR *ticketinfo)
 {
-    return(KFAILURE);
+    return(255);
 }
 
 
@@ -1018,7 +874,7 @@ config_boolean_to_int(const char *s)
  * - string resource in the leash DLL
  */
 
-BOOL
+static BOOL
 get_DWORD_from_registry(
     HKEY hBaseKey,
     char * key,
@@ -1041,33 +897,6 @@ get_DWORD_from_registry(
     return rc?FALSE:TRUE;
 }
 
-BOOL
-get_STRING_from_registry(
-    HKEY hBaseKey,
-    char * key,
-    char * value,
-    char * outbuf,
-    DWORD  outlen
-    )
-{
-    HKEY hKey;
-    DWORD dwCount;
-    LONG rc;
-
-	if (!outbuf || outlen == 0)
-		return FALSE;
-
-    rc = RegOpenKeyEx(hBaseKey, key, 0, KEY_QUERY_VALUE, &hKey);
-    if (rc)
-        return FALSE;
-
-    dwCount = outlen;
-    rc = RegQueryValueEx(hKey, value, 0, 0, (LPBYTE) outbuf, &dwCount);
-    RegCloseKey(hKey);
-
-    return rc?FALSE:TRUE;
-}
-
 static
 BOOL
 get_default_lifetime_from_registry(
@@ -1167,7 +996,7 @@ Leash_get_default_lifetime(
                            use the form 'ticket_lifetime = 600' where
                            the unit is assumed to be minutes.  While
                            these are technically wrong (a unit needs
-                           to be specified), we try to accomodate for
+                           to be specified), we try to accommodate for
                            this using the safe assumption that the
                            unit is seconds and tack an 's' to the end
                            and see if that works. */
@@ -1319,7 +1148,7 @@ Leash_get_default_renew_till(
                            use the form 'ticket_lifetime = 600' where
                            the unit is assumed to be minutes.  While
                            these are technically wrong (a unit needs
-                           to be specified), we try to accomodate for
+                           to be specified), we try to accommodate for
                            this using the safe assumption that the
                            unit is seconds and tack an 's' to the end
                            and see if that works. */
@@ -1888,63 +1717,6 @@ Leash_get_default_publicip(
     return 0;
 }
 
-static
-BOOL
-get_default_use_krb4_from_registry(
-    HKEY hBaseKey,
-    DWORD * result
-    )
-{
-    return get_DWORD_from_registry(hBaseKey,
-                                   LEASH_REGISTRY_KEY_NAME,
-                                   LEASH_REGISTRY_VALUE_USEKRB4,
-                                   result);
-}
-
-DWORD
-Leash_reset_default_use_krb4(
-    )
-{
-    HKEY hKey;
-    LONG rc;
-
-    rc = RegOpenKeyEx(HKEY_CURRENT_USER, LEASH_REGISTRY_KEY_NAME, 0, KEY_WRITE, &hKey);
-    if (rc)
-        return rc;
-
-    rc = RegDeleteValue(hKey, LEASH_REGISTRY_VALUE_USEKRB4);
-    RegCloseKey(hKey);
-
-    return rc;
-}
-
-DWORD
-Leash_set_default_use_krb4(
-    DWORD minutes
-    )
-{
-    HKEY hKey;
-    LONG rc;
-
-    rc = RegCreateKeyEx(HKEY_CURRENT_USER, LEASH_REGISTRY_KEY_NAME, 0,
-                        0, 0, KEY_WRITE, 0, &hKey, 0);
-    if (rc)
-        return rc;
-
-    rc = RegSetValueEx(hKey, LEASH_REGISTRY_VALUE_USEKRB4, 0, REG_DWORD,
-                       (LPBYTE) &minutes, sizeof(DWORD));
-    RegCloseKey(hKey);
-
-    return rc;
-}
-
-DWORD
-Leash_get_default_use_krb4(
-    )
-{
-    return 0;	/* don't use krb4 */
-}
-
 static
 BOOL
 get_hide_kinit_options_from_registry(
@@ -2011,12 +1783,12 @@ Leash_get_hide_kinit_options(
     hmLeash = GetModuleHandle(LEASH_DLL);
     if (hmLeash)
     {
-        char use_krb4[80];
+        char hide_kinit_options[80];
         if (LoadString(hmLeash, LSH_DEFAULT_DIALOG_KINIT_OPT,
-                       use_krb4, sizeof(use_krb4)))
+                       hide_kinit_options, sizeof(hide_kinit_options)))
         {
-            use_krb4[sizeof(use_krb4) - 1] = 0;
-            return atoi(use_krb4);
+            hide_kinit_options[sizeof(hide_kinit_options) - 1] = 0;
+            return atoi(hide_kinit_options);
         }
     }
     return 0;	/* hide unless otherwise indicated */
@@ -2090,12 +1862,12 @@ Leash_get_default_life_min(
     hmLeash = GetModuleHandle(LEASH_DLL);
     if (hmLeash)
     {
-        char use_krb4[80];
+        char life_min[80];
         if (LoadString(hmLeash, LSH_DEFAULT_DIALOG_LIFE_MIN,
-                       use_krb4, sizeof(use_krb4)))
+                       life_min, sizeof(life_min)))
         {
-            use_krb4[sizeof(use_krb4) - 1] = 0;
-            return atoi(use_krb4);
+            life_min[sizeof(life_min) - 1] = 0;
+            return atoi(life_min);
         }
     }
     return 5; 	/* 5 minutes */
@@ -2167,12 +1939,12 @@ Leash_get_default_life_max(
     hmLeash = GetModuleHandle(LEASH_DLL);
     if (hmLeash)
     {
-        char use_krb4[80];
+        char life_max[80];
         if (LoadString(hmLeash, LSH_DEFAULT_DIALOG_LIFE_MAX,
-                       use_krb4, sizeof(use_krb4)))
+                       life_max, sizeof(life_max)))
         {
-            use_krb4[sizeof(use_krb4) - 1] = 0;
-            return atoi(use_krb4);
+            life_max[sizeof(life_max) - 1] = 0;
+            return atoi(life_max);
         }
     }
     return 1440;
@@ -2244,12 +2016,12 @@ Leash_get_default_renew_min(
     hmLeash = GetModuleHandle(LEASH_DLL);
     if (hmLeash)
     {
-        char use_krb4[80];
+        char renew_min[80];
         if (LoadString(hmLeash, LSH_DEFAULT_DIALOG_RENEW_MIN,
-                       use_krb4, sizeof(use_krb4)))
+                       renew_min, sizeof(renew_min)))
         {
-            use_krb4[sizeof(use_krb4) - 1] = 0;
-            return atoi(use_krb4);
+            renew_min[sizeof(renew_min) - 1] = 0;
+            return atoi(renew_min);
         }
     }
     return 600;  	/* 10 hours */
@@ -2321,94 +2093,17 @@ Leash_get_default_renew_max(
     hmLeash = GetModuleHandle(LEASH_DLL);
     if (hmLeash)
     {
-        char use_krb4[80];
+        char renew_max[80];
         if (LoadString(hmLeash, LSH_DEFAULT_DIALOG_RENEW_MAX,
-                       use_krb4, sizeof(use_krb4)))
+                       renew_max, sizeof(renew_max)))
         {
-            use_krb4[sizeof(use_krb4) - 1] = 0;
-            return atoi(use_krb4);
+            renew_max[sizeof(renew_max) - 1] = 0;
+            return atoi(renew_max);
         }
     }
     return 60 * 24 * 30;
 }
 
-static
-BOOL
-get_lock_file_locations_from_registry(
-    HKEY hBaseKey,
-    DWORD * result
-    )
-{
-    return get_DWORD_from_registry(hBaseKey,
-                                   LEASH_REGISTRY_KEY_NAME,
-                                   LEASH_REGISTRY_VALUE_LOCK_LOCATION,
-                                   result);
-}
-
-DWORD
-Leash_reset_lock_file_locations(
-    )
-{
-    HKEY hKey;
-    LONG rc;
-
-    rc = RegOpenKeyEx(HKEY_CURRENT_USER, LEASH_REGISTRY_KEY_NAME, 0, KEY_WRITE, &hKey);
-    if (rc)
-        return rc;
-
-    rc = RegDeleteValue(hKey, LEASH_REGISTRY_VALUE_LOCK_LOCATION);
-    RegCloseKey(hKey);
-
-    return rc;
-}
-
-DWORD
-Leash_set_lock_file_locations(
-    DWORD onoff
-    )
-{
-    HKEY hKey;
-    LONG rc;
-
-    rc = RegCreateKeyEx(HKEY_CURRENT_USER, LEASH_REGISTRY_KEY_NAME, 0,
-                        0, 0, KEY_WRITE, 0, &hKey, 0);
-    if (rc)
-        return rc;
-
-    rc = RegSetValueEx(hKey, LEASH_REGISTRY_VALUE_LOCK_LOCATION, 0, REG_DWORD,
-                       (LPBYTE) &onoff, sizeof(DWORD));
-    RegCloseKey(hKey);
-
-    return rc;
-}
-
-DWORD
-Leash_get_lock_file_locations(
-    )
-{
-    HMODULE hmLeash;
-    DWORD result;
-
-    if (get_lock_file_locations_from_registry(HKEY_CURRENT_USER, &result) ||
-        get_lock_file_locations_from_registry(HKEY_LOCAL_MACHINE, &result))
-    {
-        return result;
-    }
-
-    hmLeash = GetModuleHandle(LEASH_DLL);
-    if (hmLeash)
-    {
-        char lock_file_locations[80];
-        if (LoadString(hmLeash, LSH_DEFAULT_DIALOG_LOCK_LOCATION,
-                       lock_file_locations, sizeof(lock_file_locations)))
-        {
-            lock_file_locations[sizeof(lock_file_locations) - 1] = 0;
-            return atoi(lock_file_locations);
-        }
-    }
-    return 0;
-}
-
 static
 BOOL
 get_default_uppercaserealm_from_registry(
@@ -2651,7 +2346,6 @@ Leash_reset_defaults(void)
     Leash_reset_default_noaddresses();
     Leash_reset_default_proxiable();
     Leash_reset_default_publicip();
-    Leash_reset_default_use_krb4();
     Leash_reset_hide_kinit_options();
     Leash_reset_default_life_min();
     Leash_reset_default_life_max();
